@@ -1,39 +1,50 @@
 use AppConfig::Std;
+use Data::Dumper;
+use File::Find;
 use Net::FTP::Common;
 use strict;
 
-my $config = AppConfig::Std->new( { CASE=>1 } );
 
-my $site = 'urth_';
+# 
+# get connection info
+#
+
+my $config = AppConfig::Std->new( { CASE=>1 } );
+my $sit  e = 'urth_';
 $config->define("$site$_") for qw(User Pass Host RemoteDir Type);
 $config->file('/Users/metaperl/.appconfig');
-
 my %urth = $config->varlist("^$site", 1);
 
-use Data::Dumper;
-
-
+#
+# setup Net::FTP::Common object
+#
 our %netftp_cfg =
     (Debug => 1, Timeout => 120);
 
 my $ez = Net::FTP::Common->new(\%urth, %netftp_cfg);
 
-
-
-use File::Find;
-
+#
+# traverse directory tree, uploading files which *dont* exist
+# this script is now a mdtm checker. that's for later
+#
 find(\&wanted, "/Users/metaperl/Documents");
 
+#
+# convert local absolute paths to remote absolute paths
+#
 sub remotedir {
-    
     my $dir = shift;
     $dir =~ s{Users/metaperl}{home/metaperl/rsync};
     $dir
 
 }
 
+
 sub unwanted {
-    shift =~ /(.FBCIndex)/;
+    my $file = shift;
+    return 1 if $file =~ /.DS_Store/;
+    return 1 if $file =~ /.FBC/;
+    return 1 if $file =~ /\.mp3$/;
 }
 
 our %mkdir;
@@ -46,7 +57,7 @@ sub wanted {
     my $ld =  $File::Find::dir;
     my $rd = remotedir $ld;
 
-    last if unwanted($lf);
+    last if unwanted($File::Find::name);
 
     warn "ld: $ld lf: $lf rd: $rd";
 
@@ -56,12 +67,13 @@ sub wanted {
       $mkdir{$rd}++;
     }
     
-    warn "ez->send(LocalFile => $lf, LocalDir => $ld)";
-
     if ($ez->exists(RemoteFile => $lf, RemoteDir => $rd)) {
-      warn "$lf already there.. skipping";
+      warn "$lf already there in $rd... skipping";
       return;
     }
 
+    warn "ez->send(LocalFile => $lf, LocalDir => $ld)";
     $ez->send(LocalFile => $lf, LocalDir => $ld);
 }
+
+
